class PostsController < ApplicationController
  before_filter :check_login

  # GET /posts
  # GET /posts.xml
  def index
    @posts = Post.find(:all)

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @posts }
    end
  end

  def show
    p = Post.find(params[:id])
    n = p.gen_name
    f = File.join(CHROOT_DIR,LATEX_BUILD_DIR,n,n+"_out.pdf")
    if !p.public_pdf
      check_user(p) { return }
    end
    send_file f,:filename=>p.file+".pdf",:type=>'application/pdf',:x_sendfile=>true
  end

  def tex
    p = Post.find(params[:id])
    n = p.gen_name
    f = File.join(CHROOT_DIR,LATEX_BUILD_DIR,n,n+".tex")
    if !p.public_tex
       check_user(p) { return }
    end
    send_file f,:filename=>p.file+".tex",:type=>'text/tex',:x_sendfile=>true
  end

  def log
    p = Post.find(params[:id])
    check_user(p) { return }
    n = p.gen_name
    f = File.join(CHROOT_DIR,LATEX_BUILD_DIR,n,n+".log")
    send_file f,:filename=>p.file+".log",:type=>'text/plain'
  end

  def new
    #first, find user
    if params[:new_filename]==""
       redirect_to :controller=>:brew,:action=>:list
    end
    @user = User.find session[:id]
    cont = ""
    if params[:temp_id]
       @temp = Template.find params[:temp_id]
       cont = @temp.template
    end
    @post = Post.new :file => params[:new_filename],
    	     	     :cont => cont
    @user.posts << @post #adding the post to the user
    if @post.save
       flash[:notice] = "Success creating file " + @post.file
       redirect_to :controller=>:brew,:action=>:edit,:id=>@post.id
    else
       flash[:error] = "Failure in creating file"
       redirect_to :controller=>:brew,:action=>:list
    end
  end

  def add_user
    @post = Post.find params[:id]
    check_user(@post) { return }
    if params[:alias]
       @user = User.find_by_name params[:alias]
    elsif params[:id_url]
       @user = User.find_by_identity_url params[:id_url]
    end

    user_pres = true
    begin
      @post.users.find @user.id
    rescue ActiveRecord::RecordNotFound
      user_pres = false
    end
    if user_pres
      flash.now[:warning] = "Already added to this document"
      return
    end
    @post.users << @user
    if @post.save
      flash.now[:notice] = "Success: adding user to document"
    else
      flash.now[:error] = "Failure: adding user to document"
    end
  end

  def rm_user
    @post = Post.find params[:id]
    check_user(@post) { return }
    begin
      @up = UserPost.find :first, :conditions=>{"post_id"=>params[:id],"user_id"=>params[:user]}
    rescue ActiveRecord::RecordNotFound
      flash.now[:error] = "User is not associated with the document."
    end
    if USER_RM_TIMEOUT>(Time.now.to_i - @up.created_at.to_i)
       @up.destroy
       flash.now[:warning] = "User successfully removed"
    else
       flash.now[:error] = "User has been associated with document for too long."
    end
  end

  def update
   #assume save button disabled when no file
   @post = Post.find(params[:id])
   check_user(@post) { return }
   @post[:cont] = params[:post][:cont]
   if @post.save
      render :nothing => true
   else
      flash.now[:error] =  "Uh Oh: Save Failed"
   end
  end

  def public_config
    @post = Post.find(params[:id])
    check_user(@post) { return }
    @post[:public_pdf] = params[:public_pdf]
    @post[:public_tex] = params[:public_tex]
    if @post.save
       flash.now[:notice] = "Preferences saved successfully"
    else
       flash.now[:error] = "Preferences not saved"
    end
  end

  #seperated out the pdf generation and saving of file
  def generate
      #assume saved?
      @post = Post.find(params[:id])
      check_user(@post) { return }
      if 4>(Time.now.to_i - @post.last_gen_at.to_i)
      	 flash.now[:warning] = "Too soon after last pdf generation: wait #{4-(Time.now.to_i - @post.last_gen_at.to_i)} more seconds"
	 return
      end
      @post.update_attribute(:last_gen_at,Time.now)

      #begin generation
      name = @post.gen_name
      #check if dedicated directory is already made up for the post
      if Dir[File.join(CHROOT_DIR,LATEX_BUILD_DIR,name)][0] == nil
      	 Dir::mkdir(File.join(CHROOT_DIR,LATEX_BUILD_DIR,name)+"/")
      end
      #write to file in separate directory
      File.open(File.join(CHROOT_DIR,LATEX_BUILD_DIR,name,name+".tex"),"w") do 
        |f| 
        f.write @post[:cont]
      end
      messages = nil
      #run it in a chroot, ensure user is www-data
#      messages = cmd "schroot -c ezlo -d #{File.join(LATEX_BUILD_DIR,name)} -- pdflatex -file-line-error -interaction nonstopmode -output-directory #{File.join(LATEX_BUILD_DIR,name)} #{name+'.tex'}"
      messages = cmd "schroot -c ezlo -d #{File.join(LATEX_BUILD_DIR,name)} -- latex -file-line-error -interaction nonstopmode -output-directory #{File.join(LATEX_BUILD_DIR,name)} #{name+'.tex'}"
      cmd "schroot -c ezlo -d #{File.join(LATEX_BUILD_DIR,name)} -- dvipdf #{name+'.dvi'}"
      #need to check for errors on every level
      w = []
      e = []
      n = messages.split("\n")
      n.each do |m|
        if m.length == 0
	   next
        end
        if m[0..12] == "LaTeX Warning"
	  w << m
        end
	if m[0..1] == "! "
	  e << m
        end
      end
      if w.length > 0
        flash.now[:warning] = w.join("<br/>\n")
      end
      flash.now[:notice] = "Log File: <a href='/ezlo/posts/log/#{@post.id}'>Here</a> <br/>\n"
      if e.length > 0
        flash.now[:error] = e.join("<br/>\n")
      else
        #move it out
        cpmess = cmd "schroot -c ezlo -d #{File.join(LATEX_BUILD_DIR,name)} -- cp #{name}.pdf #{name}_out.pdf"
	if $? == 0
          flash.now[:notice] += "PDF: <a href='/ezlo/posts/#{@post.id}'>Here</a>"
	else
	  flash.now[:error] = "Error making a persistent pdf, error code #{$?.to_s}"
        end
      end
  end

  # DELETE /posts/1
  # DELETE /posts/1.xml
  def destroy
    @post = Post.find(params[:id])
    check_user(@post) { return }
    if @post.users.length == 1
       @post.destroy
    else
       @post.users.delete(User.find(session[:id]))
    end
    flash[:warning] = "File deleted"
    redirect_to :controller=>:brew, :action=>:list
  end

  protected
  def check_user(post)
    begin
      post.users.find session[:id]
    rescue Exception
    	flash[:error] = "You do not have the appropriate permissions to access this resource."
	redirect_to :controller=>:brew,:action=>:list
	yield
    end
  end
end
