require 'openid'
require 'openid/store/filesystem'

require 'uri'
require 'net/http'
require 'net/https'
require 'json'

class UsersController < ApplicationController
  def new
    id = params[:openid_url]
    begin 
        response = openid_consumer.begin id
    rescue OpenID::OpenIDError => e
    	flash[:error] = "Discovery for #{id} failed, #{e}"
	redirect_to :controller=>:brew, :action=>:index
	return
    end
    return_to = url_for :action => 'update', :only_path=>false
    realm = url_for :action => :index, :only_path=>false

    redirect_to response.redirect_url realm, return_to
    #render :text => response.html_markup(realm, return_to, false, {:id => :openid_form})
   
  end

  #do adding of new users here
  #named update, instead of complete, because it works without routing magic
  def update
      #had to steal next 2 lines from the open-id-authentication library
      params_with_path = params.reject { |key, value| request.path_parameters[key] }
      str = "#{request.protocol + request.host_with_port + ActionController::Base.relative_url_root + request.path}"
      r = openid_consumer.complete(params_with_path,str)
      if r.status == OpenID::Consumer::SUCCESS
	 @user = User.find_by_identity_url(r.identity_url)
	 if !(@user)
	    @user = User.new :identity_url=>r.identity_url, :name=>r.identity_url
	    if @user.save
	       flash.now[:notice] = "Created new user"
	    else
	       flash.now[:notice] = "Failed creating new user"
	    end
	 end 
	 reset_session
	 session[:id] = @user.id
       	 session[:id_url] = r.identity_url
	 flash.now[:Notice] = "Log in successful"
	 redirect_to :controller=>:brew,:action=>:index
	 return
      end

      flash[:error] = "Validation failed: " + r.message
      redirect_to :controller=>:brew,:action=>:index
  end

  def rpx
      rPX_API_KEY = "#####"
      
      rpx_token = params[:token]
      if rpx_token.blank?
      	 flash[:notice] = "Cancelled RPX Login"
	 redirect_to :controller=>:brew,:action=>:index
	 return
      end
      rpx = Net::HTTP.new('rpxnow.com', 443)
      rpx.use_ssl = true
      path = "/api/v2/auth_info"
      args = "apiKey=#{rPX_API_KEY}&token=#{rpx_token}"
      http_resp, response_data = rpx.post( path, args )

      if http_resp.code == "200"
         rpx_data = JSON.parse( response_data )
      else
         flash[:error] = "Well, this didn't go well"
	 redirect_to :controller=>:brew,:action=>:index
	 return
      end
      if rpx_data["stat"] != "ok"
      	 if rpx_data["err"]
	    flash[:error] = "RPX Error "+rpx_data["err"]["code"].to_s+": "+rpx_data["err"]["msg"]
	 else
	    flash[:error] = "RPX Error: Undetermined"
	 end
	 redirect_to :controller=>:brew,:action=>:index
	 return
      end
      if !rpx_data or !rpx_data["profile"]
      	 flash.now[:error] = rpx_data
      end
      id = rpx_data["profile"]["identifier"]
      @user = User.find_by_identity_url(id)
      if !(@user)
      	 name = rpx_data["profile"]["displayName"]
	 if !name
	    name = id
	 end
         @user = User.new :identity_url=>id, :name=>name
	 if @user.save
	    flash.now[:notice] = "Created new user"
	 else
	    flash.now[:error] = "Failed creating new user"
	 end
      end 
      reset_session
      session[:id] = @user.id
      session[:id_url] = id
      flash.now[:notice] = "Log in successful"
      redirect_to :controller=>:brew,:action=>:index
      return
  end

  def logout
      reset_session      
      flash[:notice] = "Logout Successful"
      redirect_to :controller=>:brew, :action=>:index
  end

  def ed_alias
      @user = User.find session[:id]
      flash.now[:notice] = "Editing alias"
  end
  
  def up_alias
      @user = User.find session[:id]
      if @user.update_attributes :name=>params[:alias]
      	 flash.now[:notice] = "Alias updated"
      else
	 flash.now[:error] = "Error: Alias not updated"
      end
  end

  # DELETE /users/1
  # DELETE /users/1.xml
  def destroy
    @user = User.find(params[:id])
    @user.destroy

    respond_to do |format|
      format.html { redirect_to(users_url) }
      format.xml  { head :ok }
    end
  end

  protected

  def openid_consumer
      store = OpenID::Store::Filesystem.new "#{RAILS_ROOT}/tmp/openid"
      @openid_consumer = OpenID::Consumer.new session,store
  end
end
